/****************************************************************************\
*  *
* File 'el_w32.cpp'   - Electron Soccer v 1.1, Win32 code                    *
* Project started     - 14:54:21, 18-11-97                                   *
* Last modifyed       - 22:11:28, 12-05-98                                   *
*                                                                            *
* Written by EXeLENZ (55:1055/2@general, 2:5030/448.1@fidonet)               *
* Win32 version written by Yoletir / FaerieFire AKA Dmitry Jemerov,          *
*                                                              2:5030/654    *
*  *
\****************************************************************************/

#include <stdarg.h>
#include <stdio.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>

#include "el_socc.h"

#include <fxRes.h>

int bActive;                             /*   㧥                 */
int bFullscreen=1;                       /* Whether we are in full screen   */
int bNoStretch=0;
int VidMaxX, VidMaxY;

HWND hElSoccWnd;

void UpdateWnd()
{
   DWORD dwStyle=GetWindowStyle (hElSoccWnd);
   dwStyle &= ~WS_POPUP;
   dwStyle |= WS_OVERLAPPED | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU;
   SetWindowLong (hElSoccWnd, GWL_STYLE, dwStyle);
   
   RECT rc;
   SetRect (&rc, 0, 0, VidMaxX, VidMaxY);
   AdjustWindowRectEx (&rc, GetWindowStyle (hElSoccWnd),
                       FALSE, GetWindowExStyle (hElSoccWnd));

   SetWindowPos (hElSoccWnd, NULL, 0, 0, rc.right-rc.left, rc.bottom-rc.top, 
                 SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);

   SetWindowPos (hElSoccWnd, HWND_NOTOPMOST, 0, 0, 0, 0,
                 SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
}

long FAR PASCAL WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   int ScanCode;
   
   switch (message) {
   case WM_ACTIVATEAPP :
      //EGLtrace ("WM_ACTIVATEAPP %i\n", wParam);
      bActive = wParam;
      break;

   case WM_MOVE :
   case WM_SIZE :
      if (!bFullscreen) {
         RECT rc;
         GetClientRect (hElSoccWnd, &rc);
         ClientToScreen (hElSoccWnd, (LPPOINT) &rc);
         ClientToScreen (hElSoccWnd, (LPPOINT) &rc+1);
         EGLsetWindowRect (&rc);
      }
      break;

   case WM_SETCURSOR :
      if (bFullscreen && bActive)
         SetCursor (NULL);
      break;

   case WM_KEYDOWN :
      //EGLtrace ("WM_KEYDOWN wParam %04hX, lParam %08lX\n", wParam, lParam);
      ScanCode = (lParam >> 16) & 0xFF;
      keymap [ScanCode] = PRESSED;
      break;

   case WM_KEYUP :
      //EGLtrace ("WM_KEYUP   wParam %04hX, lParam %08lX\n", wParam, lParam);
      ScanCode = (lParam >> 16) & 0xFF;
      keymap [ScanCode] = DEPRESSED;
      break;

   case WM_DESTROY :
      ElSoccDone();
      PostQuitMessage (0);
      break;

   }
   
   return DefWindowProc (hWnd, message, wParam, lParam);
}


int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/,
                    LPSTR /*lpCmdLine*/, int nCmdShow)
{
   /* Read the .INI file */
   int bProportional=0;
   char AppName [260];

   GetModuleFileName (NULL, AppName, 259);
   char *p=strrchr (AppName, '\\');
   strcpy (p+1, "elsocc.ini");

   if (!GetPrivateProfileInt ("ElSocc", "EnablePlasma", 1, AppName))
      NOPLASMA=YES;
   if (!GetPrivateProfileInt ("ElSocc", "EnableSound", 1, AppName))
      NOSOUND=YES;
   if (!GetPrivateProfileInt ("ElSocc", "EnableMusic", 1, AppName))
      NOMUSIC=YES;

   switch (GetPrivateProfileInt ("ElSocc", "VideoMode", 0, AppName)) {
   case 0 :
      bFullscreen=1;
      if (EGLhaveMode (320, 200, 16)) {
         bNoStretch=1;
         bProportional=0;
      }
      else bNoStretch=0;
      break;
   case 1 :
      bFullscreen=1; bProportional=1; bNoStretch=0;
      break;
   case 2 :
      bFullscreen=1; bProportional=1; bNoStretch=1;
      break;
   case 3 :
      bFullscreen=0; bNoStretch=1;
      break;
   case 4 :
      bFullscreen=0; bNoStretch=0;
      break;
   }

   GameSpeed = (float) GetPrivateProfileInt ("ElSocc", "GameSpeed", 10, AppName) / 10;

   GameTime = GetPrivateProfileInt ("ElSocc", "GameLength", 60, AppName);

   if (!bFullscreen)
   {
      HDC hDesktopDC = GetDC (0);
      int iBPP=GetDeviceCaps (hDesktopDC, BITSPIXEL);
      if (iBPP <= 8 || iBPP > 16)
      {
         MessageBox (NULL, "When run in a window, Electron Soccer requires a high-color display",
                     "Electron Soccer", MB_OK | MB_ICONERROR);
         return 0;
      }
   }
         
   WNDCLASS wc;

   // Register window class
   wc.style = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc = WndProc;
   wc.cbClsExtra = 0;
   wc.cbWndExtra = 0;
   wc.hInstance = hInstance;
   wc.hIcon = LoadIcon (hInstance, IDI_APPLICATION);
   wc.hCursor = LoadCursor (hInstance, IDC_ARROW);
   wc.hbrBackground = NULL;
   wc.lpszMenuName = NULL;
   wc.lpszClassName = "ElSocc";
   RegisterClass (&wc);
 
   // Create window
   hElSoccWnd = CreateWindowEx (
     0,
     "ElSocc",
     "Electron Soccer",
     WS_POPUP,
     0,
     0,
     GetSystemMetrics (SM_CXSCREEN),
     GetSystemMetrics (SM_CYSCREEN),
     NULL,
     NULL,
     hInstance,
     NULL);
 
   if (!hElSoccWnd) return FALSE;
   
   ShowWindow (hElSoccWnd, nCmdShow);
   UpdateWindow (hElSoccWnd);
   SetFocus (hElSoccWnd);

   if (!ResInitFile ("els.res")) {
      MessageBox (NULL, "Cannot open resource file", "Electron Soccer", MB_OK);
      return FALSE;
   }

   if (bFullscreen) {
      if (EGLhaveMode (320, 200, 16) && !bProportional) {
         bNoStretch=1;
         VidMaxX = 320; VidMaxY = 200;
      } else {
         VidMaxX = 640; VidMaxY = 480;
      }
   } else {
      if (bNoStretch) {
         VidMaxX = 320; VidMaxY = 200;
      } else {
         VidMaxX = 640; VidMaxY = 400;
      }
      UpdateWnd();
   }
   
   if (!EGLinitDirectDraw (hElSoccWnd, VidMaxX, VidMaxY, 16, bFullscreen,
                           MaxX, MaxY, bNoStretch))
      return FALSE;
   OutputDebugString ("DirectDraw initialized\n");

#ifdef USE_MIDAS
   // 樠㥬 .
   MIDASstartup();
   MIDASsetOption (MIDAS_OPTION_DSOUND_MODE, MIDAS_DSOUND_PRIMARY);
   MIDASsetOption (MIDAS_OPTION_DSOUND_HWND, (DWORD) hElSoccWnd);
#endif
   
   ElSoccMain();

   return TRUE;
}

void ProcessMessages()
{
   MSG msg;
   
   if (PeekMessage (&msg, NULL, 0, 0, PM_NOREMOVE)) {
      if (!GetMessage (&msg, NULL, 0, 0))
         return;
      TranslateMessage (&msg);
      DispatchMessage (&msg);

      while (!bActive) {
         WaitMessage();
         if (!GetMessage (&msg, NULL, 0, 0))
            return;
         TranslateMessage (&msg);
         DispatchMessage (&msg);
      }
   }
}

void delay (unsigned int __milliseconds)
{
   Sleep (__milliseconds);
}

void DeinitKbd()
{
}

/****************************************************************************\
* Function : Error                                                           *
* Desc     : 砥 ண,  稭.                                   *
* Input    : 稭                                                         *
* Output   :                                                             *
\****************************************************************************/
void Error (int cause) {

   EGLterminate ();
   switch (cause) {
   case ErrMusax :
      MessageBox (NULL, "Trouble with audio engine, sorry.", "Electron Soccer", MB_OK);
      break;
   }
   
   exit (cause);
}

/****************************************************************************\
* Function : EGLtrace                                                        *
* Desc     : ⫠筠                                                *
* Input    : ଠ⭠ ப, 㬥                                     *
* Output   :                                                              *
\****************************************************************************/
void __cdecl EGLtrace (char *fmt, ...)
{
   char buf [256];
   va_list va;

   va_start (va, fmt);
   vsprintf (buf, fmt, va);
   va_end (va);

   OutputDebugString (buf);
}
