/* $Header: /C/FaerieX/FFRC/FFRC.cpp 1     20.07.98 22:36 Yole $
   FaerieFire resource compiler: main program file
   Written by Yoletir / FaerieFire AKA Dmitry Jemerov, 2:5030/654
*/

#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define __RES_HEADER
typedef int HRESELEM;
#include <fxRes.h>

#ifdef WIN32
#define PROG_NAME "FFRCNT"
#else
#define PROG_NAME "FFRC"
#endif

// list of resource elements
struct ResElem
{
  ResDirEntry de;
  char *FileName;
  char *ResID;
  int indirect;
  ResElem *next;
};

ResHeader rh;

ResElem *reFirst=NULL, *reLast=NULL;
int reCount=0;
int CPPcomments=0;

FILE *lstf, *resf, *hf;

//
// AddResElem()
// adds elem to element list

void AddResElem (char *aFileName, char *aResID, int aIndirect) {
  if (!reFirst) {
    reFirst=new ResElem;
    reLast=reFirst;
  } else {
    reLast->next = new ResElem;
    reLast=reLast->next;
  }
  reLast->next = NULL;
  reLast->FileName = new char [strlen (aFileName)+1];
  strcpy (reLast->FileName, aFileName);
  reLast->ResID = new char [strlen (aResID)+1];
  strcpy (reLast->ResID, aResID);
  reLast->indirect = aIndirect;
  reCount++;
}

//
// WriteComment()
// Writes a single comment line to header file

void WriteComment (char *s) 
{
  if (CPPcomments)
    fprintf (hf, "// %s\n", s);
  else
    fprintf (hf, "/* %s */\n", s);
}

//
// help()
// Displays command line help

void help() 
{
  puts ("\nUsage: "PROG_NAME" resname [options]");
  puts ("  resname.lst - resource list file");
  puts ("  resname.ffr - output resource file");
  puts ("  resname.h   - output header file\n");
  puts ("Options:");
  puts ("  /bind <filename> - bind resource to the end of filename");
  puts ("  /O <directory>   - specify resource output directory");
  puts ("  /cpp             - C++ style header file comments\n");
  puts (PROG_NAME" /? - help");
  exit (1);
}

/////////////////////////////////////////////////////////////////////////////
// main()

int main (int argc, char *argv[]) 
{
  puts ("FaerieFire Resource Compiler");
  puts ("Written by Yoletir / FaerieFire AKA Dmitry Jemerov, 2:5030/654");
  
  if (argc < 2) help();
  if ((!strcmp (argv [1], "/?")) || (!strcmp (argv [1], "-?"))) help();

  // prepare filename
  char fname [255], fullname [255], adrive [_MAX_DRIVE], adir [_MAX_DIR];
  char in_dir [255], out_dir [255];

  _splitpath (argv [1], adrive, adir, fname, NULL);
  strcpy (in_dir, adrive);
  strcat (in_dir, adir);
  if (!strlen (in_dir))
  {
    _splitpath (argv [0], adrive, adir, NULL, NULL);
    strcpy (in_dir, adrive);
    strcat (in_dir, adir);
  }
  strcpy (out_dir, in_dir);

  // parse other options
  int bind=0;
  char bindfname [255];
  long BindPos;
  for (int arg=2; arg<argc; arg++) 
  {
    if (argv [arg][0] == '-') argv [arg][0] = '/';
    if (!strcmpi (argv [arg], "/BIND")) 
    {
      if (arg==argc-1) 
      {
        puts ("[Err] /BIND filename not specified");
        exit (1);
      }
      bind=1;
      strcpy (bindfname, argv [++arg]);
    }
    else if (!strcmpi (argv [arg], "/O"))
    {
      if (arg == argc-1)
      {
        puts ("[Err] /O directory not specified");
        exit (1);
      }
      strcpy (out_dir, argv [++arg]);
      if (out_dir [strlen (out_dir) - 1] != '\\')
        strcat (out_dir, "\\");
    }
    else if (!strcmpi (argv [arg], "/CPP"))
      CPPcomments=1;
    else {
      printf ("[Err] Unknown parameter %s\n", argv [arg]);
      help();
    }
  }

  // open files
  sprintf (fullname, "%s%s.lst", in_dir, fname);
  lstf=fopen (fullname, "rt");
  if (!lstf) 
  {
    printf ("[Err] Cannot open resource list file %s\n", fullname);
    exit (1);
  }
  
  if (!bind) 
  {
    sprintf (fullname, "%s%s.ffr", out_dir, fname);
    resf=fopen (fullname, "wb"); //TODO: ask for overwrite
    if (!resf) 
    {
      printf ("[Err] Cannot open output resource file %s\n", fullname);
      exit (1);
    }
    BindPos = 0;
  } 
  else {
    if (access (bindfname, 0))
      puts ("[Warn] Bind file doesn't exist");
    resf=fopen (bindfname, "r+b");
    if (!resf) 
    {
      printf ("[Err] Cannot open bind file %s\n", bindfname);
      exit (1);
    }
    fseek (resf, 0, SEEK_END);
    BindPos = ftell (resf);
  }

  sprintf (fullname, "%s%s.h", out_dir, fname);
  hf=fopen (fullname, "wt");
  if (!hf) 
  {
    printf ("[Err] Cannot open output header file %s\n", fullname);
    exit (1);
  }

  //write header header :)
  char buf [128];
  fprintf (hf, "#ifndef __RES_HEADER\n#define __RES_HEADER\n");
  sprintf (buf, "Resource header for %s.ffr", fname);
  WriteComment (buf);
  WriteComment ("Generated by FaerieFire resource compiler");
  fprintf (hf, "\n");
  fprintf (hf, "typedef enum {\n");

  //read and parse resource list file
  printf  ("Loading resource list... ");
  char *p, *dp;
  char aFileName [80], aResID [80];
  int lstline=0, aIndirect=0;
  while (!feof (lstf)) 
  {
    lstline++;
    if (!fgets (buf, 127, lstf)) break;
    p=strrchr (buf, '\n');
    if (p) *p=NULL;
    p=buf;
    while ((*p == ' ') || (*p == '\t')) p++; //skip whitespace
    if (*p == ';') 
    {
      p++;
      WriteComment (p);
      continue;
    }
    if (*p == 0) 
    {
      fprintf (hf, "\n");
      continue;
    }
    if (*p == '!') 
    {
      aIndirect = 1;
      p++;
    } else aIndirect=0;
    dp=aFileName;
    while ((*p != ' ') && (*p != '\t') && (*p != 0)) *dp++ = *p++; //TODO: check aFileName length
    *dp=0;
    while ((*p == ' ') || (*p == '\t')) p++; //skip whitespace
    if (*p == 0) 
    {
      printf ("[Err] Resource list file, line %i: ResID not specified\n",
              lstline);
      fcloseall();
      exit (1);
    }
    dp=aResID;
    while ((*p != ' ') && (*p != '\t') && (*p != 0)) *dp++ = *p++; //TODO: check aResID length
    *dp = 0;
    AddResElem (aFileName, aResID, aIndirect);
    //write .H file entry
    if (reCount > 1)
      fprintf (hf, ",\n");
    fprintf (hf, "  %s = %i", aResID, reCount-1);
  }
  if (reCount == 0) 
  {
    puts ("[Err] No resources specified in list file");
    fcloseall();
    exit (1);
  }
  fprintf (hf, "\n} HRESELEM;\n\n#endif  /* __RES_HEADER */\n");
  printf ("%i resources.\n", reCount);

  //write resource header
  if (!bind)
    puts ("Writing resource file...");
  else
    puts ("Writing bound resource file...");
  strncpy (rh.Sign, "FaeR", 4);
  rh.DirOffset=0;
  rh.ResCount=reCount;
  fwrite (&rh, sizeof (rh), 1, resf);

  //write resource elements
  FILE *ref; // Resource Element File
  char *ResBuf=NULL;
  long ResBufSize=0, CurOffset=ftell (resf), ResLen, CurRes;
  ResElem *reCur;
  for (reCur=reFirst, CurRes=0; reCur; reCur=reCur->next, CurRes++) 
  {
    ref=fopen (reCur->FileName, "rb");
    if (!ref) 
    {
      printf ("[Err] Cannot open resource element file %s\n", reCur->FileName);
      fcloseall();
      exit (1);
    }

    //set up ResDirEntry struct
    fseek (ref, 0, SEEK_END);
    ResLen=ftell (ref);
    fseek (ref, 0, SEEK_SET);
    reCur->de.Offset = CurOffset;
    if (reCur->indirect) 
    {
      reCur->de.RealSize = 0;
      // convert FileName to full path
      _fullpath (buf, reCur->FileName, 127);
      delete reCur->FileName;
      reCur->FileName = new char [strlen (buf)+1];
      strcpy (reCur->FileName, buf);
      reCur->de.PackedSize = strlen (reCur->FileName)+1;
      reCur->de.Type = 0xFF00;
    } 
    else {
      reCur->de.RealSize = reCur->de.PackedSize = ResLen;
      reCur->de.Type = 0;
    }

    if (reCur->indirect)
      fwrite (reCur->FileName, strlen (reCur->FileName)+1, 1, resf);
    else {
      //append ref to resf
      if (ResBufSize < ResLen)  //buffer too small, reallocate
      {
        if (ResBuf) delete ResBuf;
        ResBufSize = ResLen;
        ResBuf = new char [ResBufSize];
        if (!ResBuf) 
        {
          puts ("[Err] Cannot allocate resource buffer");
          fcloseall(); exit (1);
        }
      }
      fread (ResBuf, ResLen, 1, ref);
      fwrite (ResBuf, ResLen, 1, resf);
    }
    CurOffset = ftell (resf);
    fclose (ref);
  }

  //fixup and write header
  rh.DirOffset = ftell (resf);
  fseek (resf, BindPos, SEEK_SET);
  fwrite (&rh, sizeof (rh), 1, resf);
  fseek (resf, 0, SEEK_END);

  //write directory
  for (reCur=reFirst; reCur; reCur=reCur->next)
    fwrite (&reCur->de, sizeof (ResDirEntry), 1, resf);

  if (bind)
    fwrite (&BindPos, 4, 1, resf); //write bound position

  //terminate
  fcloseall();
  delete ResBuf;
  return 0;
}
